/*
* BinaryClock.c
*
* Created: 19.07.2012 22:51:55
*  Author: Milka
*/

#define F_CPU 4000000UL

/*
* Latch fr Sekunden Port B3
* Latch fr Minuten Port B4
* Latch fr Stunden Port B5
*
* LEDs fr die Einerstelle: Port C0 bis C3
* LEDs fr die Zehnerstelle: Port B0 bis B2
*
* DCF: Pin D2 (ATMega8 INT0)
*/

#define Latch PORTB

#define LatchS PORTB3
#define LatchM PORTB4
#define LatchH PORTB5

#define LED1 PORTC // LEDs fr Einerstelle
#define LED10 PORTB //LEDs fr Zehnerstelle

#define DCFPin PIND2

#include <avr/io.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include "uart.h"

volatile unsigned char subCounter;
volatile unsigned char seconds;
volatile unsigned char minutes;
volatile unsigned char hours;

unsigned char upper;
unsigned char lower;

unsigned char current_bit = 0;

unsigned int sent_length = 0;
unsigned int unsent_length = 0;

unsigned char tMin = 0;
unsigned char tHour = 0;

unsigned char tMin_old = 0;
unsigned char tHour_old = 0;

unsigned char parity = 0;

unsigned char invalidSignal = 0;


int main(void) {
	
	uart_init();
	
	//Settings
	
	DDRB = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFB;
	
	PORTB = 0xFF;
	PORTC = 0xFF;
	PORTD = 0xFF;
	
	Latch = 0x07;
	
	subCounter = 0;
	seconds = 0;
	minutes = 0;
	hours = 0;
	
	tMin_old = 0;
	tHour_old = 0;
	
	//Clock Timer
	OCR1A = 39999; // 40000 Cycles minus 1
	TCCR1B = (1 << WGM12) | (1 << CS10);
	TIMSK = (1 << OCIE1A);
	TCNT1 = 0;
	//
	
	//DCF Timer
	TCCR0 = (1<<CS01);
	TCNT0 = 6;
	TIMSK |= (1<<TOIE0);
	//
	
	sei();
	
	// Main
	
	uart_sendString("Init Done!");
	uart_newLine();
	
	while(1) {
		
		if (subCounter >= 100) {
			subCounter = 0;
			seconds++;
			if (seconds == 60) {
				seconds = 0;
				minutes++;
				if (minutes == 60) {
					minutes = 0;
					hours++;
					if (hours == 24) {
						hours = 0;
					}
				}
			}
		}
		
		display_seconds();
		_delay_us(100);
		display_minutes();
		_delay_us(100);
		display_hours();
		_delay_us(100);
	}
}

// Timer 1 (Clock Timer; 100 Interrupts / Second)
ISR(TIMER1_COMPA_vect) {
	subCounter++;
}

// Umwandeln der Sekunden/Minuten/Stunden in einzelne Variablen (lower/upper) fr Einer- und Zehnerstelle
void convert(unsigned char value) {
	upper = 0;
	lower = 0;
	
	while (value >= 10) {
		upper += 1;
		value -= 10;
	}
	lower = value;
	return;
}

// Anzeigen der Sekunden
void display_seconds(void) {
	convert(seconds);
	
	LED1 = (LED1 | 0x0F) & ~lower; // 0x0F = 0b00001111
	LED10 = (LED10 | 0x07) & ~upper; // 0x07 = 0b00000111
	Latch = (Latch & 0x07) | (1 << LatchS);
	Latch = (Latch & 0x07);
}

// Anzeigen der Minuten
void display_minutes(void) {
	convert(minutes);
	LED1 = (LED1 | 0x0F) & ~lower; // 0x0F = 0b00001111
	LED10 = (LED10 | 0x07) & ~upper; // 0x07 = 0b00000111
	Latch = (Latch & 0x07) | (1 << LatchM);
	Latch = (Latch & 0x07);
}

// Anzeigen der Stunden
void display_hours(void) {
	convert(hours);
	LED1 = (LED1 | 0x0F) & ~lower; // 0x0F = 0b00001111
	LED10 = (LED10 | 0x07) & ~upper; // 0x07 = 0b00000111
	Latch = (Latch & 0x07) | (1 << LatchH);
	Latch = (Latch & 0x07);
}

// DCF Funktionen

// Timer 0 (2000 Overflows / Second)
ISR (TIMER0_OVF_vect) {
	
	TCNT0 = 6;
	
	if (PIND & (1<<DCFPin)) {
		// 59. Bit gefunden, da mehr als 1,25s "nichts" gesendet wurde.
		if (unsent_length >= 2000) {
			uart_sendString("Bit 59!");
			uart_newLine();
			dcf_TestAndSet();
			clear_vars();
			uart_sendString("new Old: ");
			uart_sendInt(tHour_old);
			uart_sendChar(':');
			uart_sendInt(tMin_old);
			uart_newLine();
		}
		unsent_length = 0;
		sent_length++;
	} else {
		if (sent_length >= 110 && sent_length < 320) {
			dcf_exec(0);
			uart_sendString("Bit ");
			uart_sendInt(current_bit - 1);
			uart_sendString(": 0");
			uart_newLine();
		} else if (sent_length >= 320) {
			dcf_exec(1);
			uart_sendString("Bit ");
			uart_sendInt(current_bit - 1);
			uart_sendString(": 1");
			uart_newLine();
		}
		sent_length = 0;
		unsent_length++;
	}
	
}

void clear_vars(void) {
	tMin = 0;
	tHour = 0;
	parity = 0;
	invalidSignal = 0;
	current_bit = 0;
}

// Prfung, ob der alte Wert (nach Hochzhlen um eine Minute) dem neuen entspricht
void dcf_TestAndSet(void) {
	
	unsigned char tMin_old_x = tMin_old;		//Hochzhlen der zuletzt empfangenen Zeit um eine Minute / Vergleich der hochgezhlten Zeit mit der neu empfangenen
	unsigned char tHour_old_x = tHour_old;
	
	uart_sendString("Invalid: ");
	uart_sendInt(invalidSignal);
	uart_newLine();
	
	uart_sendString("Time: ");
	uart_sendInt(tHour);
	uart_sendChar(':');
	uart_sendInt(tMin);
	uart_newLine();
	
	uart_sendString("Old_x: ");
	uart_sendInt(tHour_old_x);
	uart_sendChar(':');
	uart_sendInt(tMin_old_x);
	uart_newLine();
	
	tMin_old_x++;
	if (tMin_old_x == 60) {
		tMin_old_x = 0;
		tHour_old_x++;
		if (tHour_old_x == 24) {
			tHour_old_x = 0;
		}
	}
	
	if (tMin > 59 || tHour > 23 || invalidSignal == 1) {
		uart_sendString("Invalid Time");
		uart_newLine();
		tMin_old = tMin_old_x;
		tHour_old = tHour_old_x;
		return;
	}
	
	tMin_old = tMin;
	tHour_old = tHour;
	
	uart_sendString("Check Time");
	uart_newLine();
	
	if (tHour_old_x == tHour) {
		if (tMin_old_x == tMin) {
			uart_sendString("Set Time");
			uart_newLine();
			hours = tHour;
			minutes = tMin;
			seconds = 0;
		} else {
			uart_sendString("Min Failed");
			uart_newLine();
		}
	} else {
		uart_sendString("Hour Failed");
		uart_newLine();
	}
	
	uart_sendString("End of Test!");
	uart_newLine();
	
	return;
}

// Auswertung des DCF-Signals (Array dcf[60])
void dcf_exec(unsigned char value) {
	
	if (current_bit == 0 || (current_bit >= 20 && current_bit <= 35)) {
		if (current_bit == 0 && value == 1) {
			invalidSignal = 1;
		} else if (current_bit == 20 && value == 0) {
			invalidSignal = 1;
		} else if (current_bit == 21 && value == 1) {
			tMin += 1;
			parity++;
		} else if (current_bit == 22 && value == 1) {
			tMin += 2;
			parity++;
		} else if (current_bit == 23 && value == 1) {
			tMin += 4;
			parity++;
		} else if (current_bit == 24 && value == 1) {
			tMin += 8;
			parity++;
		} else if (current_bit == 25 && value == 1) {
			tMin += 10;
			parity++;
		} else if (current_bit == 26 && value == 1) {
			tMin += 20;
			parity++;
		} else if (current_bit == 27 && value == 1) {
			tMin += 40;
			parity++;
		} else if (current_bit == 28) {
			if (value == 0 && (parity == 1 || parity == 3 || parity == 5 || parity == 7)) {
				invalidSignal = 1;
			} else if (value == 1 && (parity == 0 || parity == 2 || parity == 6 || parity == 8)) {
				invalidSignal = 1;
			}
			parity = 0;
		} else if (current_bit == 29 && value == 1) {
			tHour += 1;
			parity++;
		} else if (current_bit == 30 && value == 1) {
			tHour += 2;
			parity++;
		} else if (current_bit == 31 && value == 1) {
			tHour += 4;
			parity++;
		} else if (current_bit == 32 && value == 1) {
			tHour += 8;
			parity++;
		} else if (current_bit == 33 && value == 1) {
			tHour += 10;
			parity++;
		} else if (current_bit == 34 && value == 1) {
			tHour += 20;
			parity++;
		} else if (current_bit == 35) {
			if (value == 0 && (parity == 1 || parity == 3 || parity == 5 || parity == 7)) {
				invalidSignal = 1;
			} else if (value == 1 && (parity == 0 || parity == 2 || parity == 6 || parity == 8)) {
				invalidSignal = 1;
			}
			parity = 0;
		}
	}
	
	current_bit++;
	
	return;
}